@@ -1,8 +1,8 @@
 /*************************************************************************
 ** FilePath.cpp                                                         **
 **                                                                      **
-** This file is part of dvisvgm -- the DVI to SVG converter             **
-** Copyright (C) 2005-2013 Martin Gieseking <martin.gieseking@uos.de>   **
+** This file is part of dvisvgm -- a fast DVI to SVG converter          **
+** Copyright (C) 2005-2019 Martin Gieseking <martin.gieseking@uos.de>   **
 **                                                                      **
 ** This program is free software; you can redistribute it and/or        **
 ** modify it under the terms of the GNU General Public License as       **
@@ -18,12 +18,11 @@
 ** along with this program; if not, see <http://www.gnu.org/licenses/>. **
 *************************************************************************/
 
-#include <config.h>
 #include <cctype>
-#include "FilePath.h"
-#include "FileSystem.h"
-#include "MessageException.h"
-#include "macros.h"
+#include "FilePath.hpp"
+#include "FileSystem.hpp"
+#include "MessageException.hpp"
+#include "utility.hpp"
 
 using namespace std;
 
@@ -37,11 +36,11 @@ static string& single_slashes (string &str) {
 }
 
 
-#ifdef __WIN32__
+#ifdef _WIN32
 static char strip_drive_letter (string &path) {
 	char letter = 0;
 	if (path.length() >= 2 && path[1] == ':' && isalpha(path[0])) {
-		letter = tolower(path[0]);
+		letter = path[0];
 		path.erase(0, 2);
 	}
 	return letter;
@@ -54,7 +53,7 @@ static char adapt_current_path (string &path, char target_drive) {
 			if (target_drive == 0)
 				target_drive = current_drive;
 			if (path.empty() || path[0] != '/') {
-				if (FileSystem::chdir((string(1, target_drive) + ":").c_str())) {
+				if (FileSystem::chdir(string(1, target_drive) + ":")) {
 					path.insert(0, FileSystem::getcwd()+"/");
 					strip_drive_letter(path);
 				}
@@ -66,19 +65,16 @@ static char adapt_current_path (string &path, char target_drive) {
 	return target_drive;
 }
 
-
-static void tolower (string &str) {
-	for (size_t i=0; i < str.length(); ++i)
-		str[i] = tolower(str[i]);
-}
 #endif
 
 
-/** Constructs a FilePath object from a given path. Relative paths are
- *  relative to the current working directory.
- *  @param[in] path absolute or relative path to a file or directory */
-FilePath::FilePath (const string &path) {
-	init(path, !FileSystem::isDirectory(path.c_str()), FileSystem::getcwd());
+bool FilePath::Directory::operator == (const Directory &dir) const {
+#ifdef _WIN32
+	// letter case is not significant on Windows systems
+	return util::tolower(_dirstr) == util::tolower(dir._dirstr);
+#else
+	return _dirstr == dir._dirstr;
+#endif
 }
 
 
@@ -86,7 +82,32 @@ FilePath::FilePath (const string &path) {
  *  @param[in] path absolute or relative path to a file or directory
  *  @param[in] isfile true if 'path' references a file, false if a directory is referenced
  *  @param[in] current_dir if 'path' is a relative path expression it will be related to 'current_dir' */
-FilePath::FilePath (const string &path, bool isfile, string current_dir) {
+FilePath::FilePath (const string &path, bool isfile, const string &current_dir) {
+	init(path, isfile, current_dir);
+}
+
+
+/** Assigns a new path of a file or directory that already exists.
+ *  Relative paths are relative to the current working directory.
+ *  @param[in] path absolute or relative path to a file or directory */
+void FilePath::set (const string &path) {
+	set(path, !FileSystem::isDirectory(path));
+}
+
+
+/** Assigns a new path. Relative paths are relative to the current working directory.
+ *  @param[in] path absolute or relative path to a file or directory
+ *  @param[in] isfile true if 'path' references a file, false if a directory is referenced */
+void FilePath::set (const string &path, bool isfile) {
+	init(path, isfile, FileSystem::getcwd());
+}
+
+
+/** Assigns a new path. Relative paths are relative to the current working directory.
+ *  @param[in] path absolute or relative path to a file or directory
+ *  @param[in] isfile true if 'path' references a file, false if a directory is referenced
+ *  @param[in] current_dir if 'path' is a relative path expression it will be related to 'current_dir' */
+void FilePath::set (const string &path, bool isfile, const string &current_dir) {
 	init(path, isfile, current_dir);
 }
 
@@ -96,10 +117,11 @@ FilePath::FilePath (const string &path, bool isfile, string current_dir) {
  *  @param[in] isfile true if 'path' references a file, false if a directory is referenced
  *  @param[in] current_dir if 'path' is a relative path expression it will be related to 'current_dir' */
 void FilePath::init (string path, bool isfile, string current_dir) {
+	_dirs.clear();
+	_fname.clear();
 	single_slashes(path);
 	single_slashes(current_dir);
-#ifdef __WIN32__
-	tolower(path);
+#ifdef _WIN32
 	path = FileSystem::adaptPathSeperators(path);
 	_drive = strip_drive_letter(path);
 #endif
@@ -113,8 +135,7 @@ void FilePath::init (string path, bool isfile, string current_dir) {
 	}
 	if (current_dir.empty())
 		current_dir = FileSystem::getcwd();
-#ifdef __WIN32__
-	tolower(current_dir);
+#ifdef _WIN32
 	_drive = adapt_current_path(current_dir, _drive);
 #endif
 	if (!path.empty()) {
@@ -125,20 +146,20 @@ void FilePath::init (string path, bool isfile, string current_dir) {
 		else {
 			FilePath curr(current_dir, false, "/");
 			current_dir = curr.absolute();
-#ifdef __WIN32__
+#ifdef _WIN32
 			adapt_current_path(current_dir, _drive);
 #endif
 		}
 	}
 	path.insert(0, current_dir + "/");
 	string elem;
-	FORALL (path, string::const_iterator, it) {
-		if (*it == '/') {
+	for (char c : path) {
+		if (c != '/')
+			elem += c;
+		else {
 			add(elem);
 			elem.clear();
 		}
-		else
-			elem += *it;
 	}
 	add(elem);
 }
@@ -149,7 +170,7 @@ void FilePath::add (const string &dir) {
 	if (dir == ".." && !_dirs.empty())
 		_dirs.pop_back();
 	else if (dir.length() > 0 && dir != ".")
-		_dirs.push_back(dir);
+		_dirs.emplace_back(dir);
 }
 
 
@@ -200,14 +221,13 @@ string FilePath::basename () const {
  *  @return the absolute path string */
 string FilePath::absolute (bool with_filename) const {
 	string path;
-	FORALL (_dirs, ConstIterator, it) {
-		path += "/" + *it;
-	}
+	for (const Directory &dir : _dirs)
+		path += "/" + string(dir);
 	if (path.empty())
 		path = "/";
 	if (with_filename && !_fname.empty())
 		path += "/"+_fname;
-#ifdef __WIN32__
+#ifdef _WIN32
 	if (_drive)
 		path.insert(0, string(1, _drive) + ":");
 #endif
@@ -225,25 +245,25 @@ string FilePath::absolute (bool with_filename) const {
 string FilePath::relative (string reldir, bool with_filename) const {
 	if (reldir.empty())
 		reldir = FileSystem::getcwd();
-#ifdef __WIN32__
+#ifdef _WIN32
 	adapt_current_path(reldir, _drive);
 #endif
 	if (reldir[0] != '/')
 		return absolute();
 	FilePath rel(reldir, false);
 	string path;
-#ifdef __WIN32__
-	if (rel._drive && _drive && rel._drive != _drive)
+#ifdef _WIN32
+	if (rel._drive && _drive && tolower(rel._drive) != tolower(_drive))
 		path += string(1, _drive) + ":";
 #endif
-	ConstIterator i = _dirs.begin();
-	ConstIterator j = rel._dirs.begin();
-	while (i != _dirs.end() && j != rel._dirs.end() && *i == *j)
-		++i, ++j;
-	for (; j != rel._dirs.end(); ++j)
+	auto it1 = _dirs.begin();
+	auto it2 = rel._dirs.begin();
+	while (it1 != _dirs.end() && it2 != rel._dirs.end() && *it1 == *it2)
+		++it1, ++it2;
+	for (; it2 != rel._dirs.end(); ++it2)
 		path += "../";
-	for (; i != _dirs.end(); ++i)
-		path += *i + "/";
+	for (; it1 != _dirs.end(); ++it1)
+		path += string(*it1) + "/";
 	if (!path.empty())
 		path.erase(path.length()-1, 1);  // remove trailing slash
 	if (with_filename && !_fname.empty()) {
@@ -256,3 +276,17 @@ string FilePath::relative (string reldir, bool with_filename) const {
 	return single_slashes(path);
 }
 
+
+/** Return the absolute or relative path whichever is shorter.
+*  @param[in] reldir absolute path to a directory
+*  @param[in] with_filename if false, the filename is omitted */
+string FilePath::shorterAbsoluteOrRelative (string reldir, bool with_filename) const {
+	string abs = absolute(with_filename);
+	string rel = relative(reldir, with_filename);
+	return abs.length() < rel.length() ? abs : rel;
+}
+
+
+bool FilePath::exists () const {
+	return empty() ? false : FileSystem::exists(absolute());
+}
\ No newline at end of file
